/*   trivoprob.ml: Maximum Likelihood Model for Trivariate Ordered Probit
**               - all 3 outcomes {OH, SoL, FS} have 3 categories {worse, same, better}
**               - HILDA W7 relative SWB in retirement
**               
**   Garry Barrett.
**   9th  May 2012
**   University of Sydney
*/

library maxlik,pgraph;
#include maxlik.ext;
#include pgraph.ext;
maxset;


/* Name of the data file  */
load dat[1344,31]=f:\work\SWBRetire\data\stockGD.dat;

/* Load starting values */
/* load theta0=d:\work\swbretire\gauss\model6sv; */

load theta0[99,1]=f:\work\swbretire\gauss\model5.sv;

_max_ParNames = {
    age45541, age55591, age60641, age70741, age75791, age80p_1,
    ret5p__1, female_1, married1, rural__1,
    edyr11_1, edctdp_1, edbapg_1, homeown1,
    mr_apa_1, mr_jobl1, mr_wstr1, mr_ohea1, mr_phea1, mr_othh1,
    mr_pret1, mr_time1, mr_othr1, forced_1, Ey_mlss1, Ey_less1,
    Ey_more1, Ey_mmre1, Ohealw_1, Ohealb_1,

    age45542, age55592, age60642, age70742, age75792, age80p_2,
    ret5p__2, female_2, married2, rural__2,
    edyr11_2, edctdp_2, edbapg_2, homeown2,
    mr_apa_2, mr_jobl2, mr_wstr2, mr_ohea2, mr_phea2, mr_othh2,
    mr_pret2, mr_time2, mr_othr2, forced_2, Ey_mlss2, Ey_less2,
    Ey_more2, Ey_mmre2, Ohealw_2, Ohealb_2,

    age45543, age55593, age60643, age70743, age75793, age80p_3,
    ret5p__3, female_3, married3, rural__3,
    edyr11_3, edctdp_3, edbapg_3, homeown3,
    mr_apa_3, mr_jobl3, mr_wstr3, mr_ohea3, mr_phea3, mr_othh3,
    mr_pret3, mr_time3, mr_othr3, forced_3, Ey_mlss3, Ey_less3,
    Ey_more3, Ey_mmre3, Ohealw_3, Ohealb_3,

    mu11____,  mu12____,
    mu21____,  mu22____,
    mu31____,  mu32____,
    rho12___,  rho23___,   rho13___
       };

   /*     */



/*------------ GLOBALS ------------------------------*/
__title = "Trivariate Ordered Probit - HILDA Relative OH, SoL and FS";

__row=0;

_max_Algorithm=5;   /* 2=BFGS 5=BHHH algorithm */
_max_CovPar=2;   /* 3 cov of par is heterosk. consistent version */
_max_MaxIters=20;


output file=f:\work\SWBRetire\gauss\TriOProb6.out reset;

/* call maxprt(maxlik(dat,0,&lnL,theta0));  */

{theta,f0,g,h,rc}=maxlik(dat,0,&lnL,theta0);
{theta,f0,g,h,rc}=maxprt(theta,f0,g,h,rc);

__output=1;
print "-----------------------------------------";
print "LLF ==   "  f0*rows(dat);
print "-----------------------------------------";

theta;
se=diag(h)^0.5;
se;

print "-----------------------------------------";
print "The Cov matrix follows                   ";
print "-----------------------------------------";

h;

output off;
ndpclex;
end;

/*--------- Log Likelihood Function --------------------*/
proc lnL(theta,din);
     local logL,x,bt1,bt2,bt3,w,y1,y2,p12,p13,p23,n,zk,zk1,Fk,xb1,xb2,xb3,
         mu11,mu12,mu21,mu22,mu31,mu32,
         z11,z12,z21,z22,z31,z32,
         b,k,j,j2,j3;

 /* w=din[.,8];     Weight variable */

k=rows(theta);
j=round((k-9)/3);
j2=2*j;
j3=3*j;

x=din[.,2:31];    /*  Covariates   */

y1=din[.,1];      /*  outcome 3x3x3 = 27 categories  */

bt1=theta[1:j];     /*  Coefficients index 1  */
bt2=theta[j+1:j2];    /*  Coefficients index 2  */
bt3=theta[j2+1:j3];    /*  Coefficients index 3  */


mu11=theta[k-8];    /*  index 1,  Cutoff 1 */
mu12=theta[k-7];    /*  index 1,  Cutoff 2 */
mu21=theta[k-6];    /*  index 2,  Cutoff 1 */
mu22=theta[k-5];    /*  index 2,  Cutoff 2 */
mu31=theta[k-4];    /*  index 3,  Cutoff 1 */
mu32=theta[k-3];    /*  index 3,  Cutoff 2 */

/*  p12=exp(-exp(-1*theta[15]));   Correlation between error 1 & 2  */

p12=theta[k-2];
p23=theta[k-1];
p13=theta[k];


/*p12=0; p23=0; p13=0;  */

/* Store the branches of the LLF in b[.,1:27] */

n=rows(din);
b=zeros(n,27);

w=ones(n,1);     /*  Weight variable */

xb1  = x*bt1;
xb2  = x*bt2;
xb3  = x*bt3;

z11=mu11-xb1;
z12=mu12-xb1;
z21=mu21-xb2;
z22=mu22-xb2;
z31=mu31-xb3;
z32=mu32-xb3;


/* 1. Pr(1,1,1) */

b[.,1] = cdftvn(z11,z21,z31,p12,p23,p13);


/* 2. Pr(1,1,2) */

b[.,2] = cdftvn(z11,z21,z32,p12,p23,p13)-cdftvn(z11,z21,z31,p12,p23,p13);


/* 3. Pr(1,1,3) */

b[.,3] = cdfbvn(z11,z21,p12)-cdftvn(z11,z21,z32,p12,p23,p13);


/* 4. Pr(1,2,1) */
b[.,4] = cdftvn(z11,z22,z31,p12,p23,p13) - cdftvn(z11,z21,z31,p12,p23,p13);


/* 5. Pr(1,3,1) */
b[.,5] = cdfbvn(z11,z31,p13) - cdftvn(z11,z22,z31,p12,p23,p13);

/* 6. Pr(1,2,2) */
b[.,6] = cdftvn(z11,z22,z32,p12,p23,p13) - cdftvn(z11,z21,z32,p12,p23,p13)
     - cdftvn(z11,z22,z31,p12,p23,p13) + cdftvn(z11,z21,z31,p12,p23,p13);

/* 7. Pr(1,2,3) */
b[.,7] = cdfbvn(z11,z22,p12) - cdftvn(z11,z22,z32,p12,p23,p13)
      - cdfbvn(z11,z21,p12) + cdftvn(z11,z21,z32,p12,p23,p13);

/* 8. Pr(1,3,2) */
b[.,8] = cdfbvn(z11,z32,p13)- cdftvn(z11,z22,z32,p12,p23,p13)
     - cdfbvn(z11,z31,p13) + cdftvn(z11,z22,z31,p12,p23,p13);

/* 9. Pr(1,3,3) */
b[.,9] = cdfn(z11) - cdfbvn(z11,z22,p12) - cdfbvn(z11,z32,p13)
      + cdftvn(z11,z22,z32,p12,p23,p13);


/* 10. Pr(2,1,1) */
b[.,10] = cdftvn(z12,z21,z31,p12,p23,p13) - cdftvn(z11,z21,z31,p12,p23,p13);

/* 11. Pr(2,1,2) */
b[.,11] = cdftvn(z12,z21,z32,p12,p23,p13) - cdftvn(z11,z21,z32,p12,p23,p13)
      - cdftvn(z12,z21,z31,p12,p23,p13) + cdftvn(z11,z21,z31,p12,p23,p13);

/* 12. Pr(2,1,3) */
b[.,12] = cdfbvn(z12,z21,p12) - cdfbvn(z11,z21,p12)
      - cdftvn(z12,z21,z32,p12,p23,p13) + cdftvn(z11,z21,z32,p12,p23,p13);

/* 13. Pr(2,2,1) */
b[.,13] =  cdftvn(z12,z22,z31,p12,p23,p13) - cdftvn(z12,z21,z31,p12,p23,p13)
       - cdftvn(z11,z22,z31,p12,p23,p13) + cdftvn(z11,z21,z31,p12,p23,p13);

/* 14. Pr(2,2,2) */
b[.,14] =  cdftvn(z12,z22,z32,p12,p23,p13) - cdftvn(z12,z21,z32,p12,p23,p13)
       - cdftvn(z11,z22,z32,p12,p23,p13) + cdftvn(z11,z21,z32,p12,p23,p13)
       - cdftvn(z12,z22,z31,p12,p23,p13) + cdftvn(z12,z21,z31,p12,p23,p13)
       + cdftvn(z11,z22,z31,p12,p23,p13) - cdftvn(z11,z21,z31,p12,p23,p13);


/* 15. Pr(2,2,3) */
b[.,15] =  cdfbvn(z12,z22,p12) - cdfbvn(z12,z21,p12) - cdfbvn(z11,z22,p12)
       + cdfbvn(z11,z21,p12)
       - cdftvn(z12,z22,z32,p12,p23,p13) + cdftvn(z12,z21,z32,p12,p23,p13)
       + cdftvn(z11,z22,z32,p12,p23,p13) - cdftvn(z11,z21,z32,p12,p23,p13);

/* 16. Pr(2,3,1) */
b[.,16] = cdfbvn(z12,z31,p13) - cdftvn(z12,z22,z31,p12,p23,p13)
      - cdfbvn(z11,z31,p13) + cdftvn(z11,z22,z31,p12,p23,p13);

/* 17. Pr(2,3,2) */
b[.,17] =  cdfbvn(z12,z32,p13) - cdfbvn(z12,z31,p13)
       - cdftvn(z12,z22,z32,p12,p23,p13) - cdfbvn(z11,z32,p13)
       + cdftvn(z12,z22,z31,p12,p23,p13) + cdfbvn(z11,z31,p13)
       + cdftvn(z11,z22,z32,p12,p23,p13) - cdftvn(z11,z22,z31,p12,p23,p13);

/* 18. Pr(2,3,3) */
b[.,18] =  cdfn(z12) - cdfn(z11) - cdfbvn(z12,z22,p12) + cdfbvn(z11,z22,p12)
       - cdfbvn(z12,z32,p13) + cdftvn(z12,z22,z32,p12,p23,p13)
       + cdfbvn(z11,z32,p13) - cdftvn(z11,z22,z32,p12,p23,p13);


/* 19. Pr(3,1,1) */
b[.,19] =  cdfbvn(z21,z31,p23) - cdftvn(z12,z21,z31,p12,p23,p13);

/* 20. Pr(3,1,2) */
b[.,20] =  cdfbvn(z21,z32,p23) - cdfbvn(z21,z31,p23)
       - cdftvn(z12,z21,z32,p12,p23,p13) + cdftvn(z12,z21,z31,p12,p23,p13);

/* 21. Pr(3,1,3) */
b[.,21] =  cdfn(z21) - cdfbvn(z12,z21,p12) - cdfbvn(z21,z32,p23)
       + cdftvn(z12,z21,z32,p12,p23,p13);

/* 22. Pr(3,2,1) */
b[.,22] =  cdfbvn(z22,z31,p23) - cdfbvn(z21,z31,p23)
       - cdftvn(z12,z22,z31,p12,p23,p13) + cdftvn(z12,z21,z31,p12,p23,p13);

/* 23. Pr(3,2,2) */
b[.,23] =  cdfbvn(z22,z32,p23) - cdfbvn(z21,z32,p23)
       - cdftvn(z12,z22,z32,p12,p23,p13) + cdftvn(z12,z21,z32,p12,p23,p13)
       - cdfbvn(z22,z31,p23) + cdfbvn(z21,z31,p23)
       + cdftvn(z12,z22,z31,p12,p23,p13) - cdftvn(z12,z21,z31,p12,p23,p13);

/* 24. Pr(3,2,3) */
b[.,24] =  cdfn(z22) - cdfn(z21) - cdfbvn(z12,z22,p12) + cdfbvn(z12,z21,p12)
       - cdfbvn(z22,z32,p23) + cdfbvn(z21,z32,p23)
       + cdftvn(z12,z22,z32,p12,p23,p13) - cdftvn(z12,z21,z32,p12,p23,p13);

/* 25. Pr(3,3,1) */
b[.,25] =  cdfn(z31) - cdfbvn(z22,z31,p23) - cdfbvn(z12,z31,p13)
       + cdftvn(z12,z22,z31,p12,p23,p13);

/* 26. Pr(3,3,2) */
b[.,26] =   cdfn(z32) - cdfbvn(z12,z32,p13) - cdfbvn(z22,z32,p23)
        + cdftvn(z12,z22,z32,p12,p23,p13)
        - cdfn(z31) + cdfbvn(z22,z31,p23) + cdfbvn(z12,z31,p13)
        - cdftvn(z12,z22,z31,p12,p23,p13);

/* 27. Pr(3,3,3) */
b[.,27] =   1 -  cdfn(z22) - cdfn(z12) +  cdfbvn(z12,z22,p12) - cdfn(z32)
        + cdfbvn(z22,z32,p23) + cdfbvn(z12,z32,p13)
        - cdftvn(z12,z22,z32,p12,p23,p13);



/* Combining the branches of the LF    */

Fk  = (y1.==1).*b[.,1] + (y1.==2).*b[.,2] + (y1.==3).*b[.,3] +
      (y1.==4).*b[.,4] + (y1.==5).*b[.,5] + (y1.==6).*b[.,6] +
      (y1.==7).*b[.,7] + (y1.==8).*b[.,8] + (y1.==9).*b[.,9] +
      (y1.==10).*b[.,10] +
      (y1.==11).*b[.,11] + (y1.==12).*b[.,12] + (y1.==13).*b[.,13] +
      (y1.==14).*b[.,14] + (y1.==15).*b[.,15] + (y1.==16).*b[.,16] +
      (y1.==17).*b[.,17] + (y1.==18).*b[.,18] + (y1.==19).*b[.,19] +
      (y1.==20).*b[.,20] +
      (y1.==21).*b[.,21] + (y1.==22).*b[.,22] + (y1.==23).*b[.,23] +
      (y1.==24).*b[.,24] + (y1.==25).*b[.,25] + (y1.==26).*b[.,26] +
      (y1.==27).*b[.,27]
      ;


  retp(ln(Fk));
endp;

